/*
megamil.c  Version 0.2.0  Mega Millions Drawings
Copyright (C) 2002-2005  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/*
Subroutine call:
    	int select;
    	int megaball;
    	double odds;
    	double getmm(int select, int megaball);
    	odds = getmm(select,megaball);
*/

/* If the chi-square test fails, repeat your test. */
/* The chi-square usually passes in the next test. */
/* If the chi-square fails consistently, then the */
/* program has to be reviewed for correctness. */

/* The higher the number of iterations, the more */
/* likely the chi-square test will pass.  The chi-square */
/* test needs to have its categories filled with at */
/* least a 10 count in order to pass consistently. */
/* With 100000 iterations, the chi-square test passes */
/* consistently. */

/* See the documentation in mmprob.c regarding the */
/* calls to getmm.c. */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rnd.h"

#define CENTURY 10436
#define MEGABALL 52
#define FIFTYTWO 52
#define FIVE 5

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage: %s ticket\n", pgm);
   fprintf(stderr,"Where ticket is 01 02 03 04 05 06\n");
   fprintf(stderr,"Numbers 01 02 03 04 05 06 are 1 - 52\n");
   fprintf(stderr,"To see probabilities, run mmprob\n");
   fprintf(stderr,"To test quick picks,  run qpick\n");
   exit(1);
   } /* putstx */

int main(argc,argv)
int argc;
char **argv;
   {
   int i,max52,indx;
   unsigned char *p,*q,*r;
   unsigned char *gameptr;
   unsigned char *trialptr;
   unsigned char fiftytwo[64];
   unsigned char ticket[64];
   unsigned char game[16];
   unsigned char trial[16];
   int tmp;
   int ticketmm;
   int winmm;
   int totmtch,won;
   int totplay;
   int maxplay;
   double prob,dblplay;
   double winning,totprize,netearn;
   double *prizeptr;
   double prize[16];
   double odds,sum,totbingo,chisq;
   double totprob;
   double expected;
   double getmm(int select, int megaball);
   double *bingoptr;
   double bingo[16];
   p = (unsigned char *) ticket;
   q = (unsigned char *) ticket + FIFTYTWO;
   while (p < q)
      *p++ = (unsigned char) '\0';
   if (argc != 7) putstx(*argv);
   gameptr = (unsigned char *) game;
   i = 1;
   while (i <= 5)
      {
      indx = atoi(*(argv+i));
      if (indx < 1 || indx > FIFTYTWO)
	 {
	 fprintf(stderr,"megamil: ticket number %s "
	    "is invalid\n",
	    *(argv+i));
	 putstx(*argv);
	 } /* if invalid number */
      p = (unsigned char *) ticket + indx;
      if (*p != 255)
         *p = (unsigned char) 255;
      else
	 {
	 fprintf(stderr,"megamil: ticket number %s "
	    "is a duplicate\n",
	    *(argv+i));
	 putstx(*argv);
	 } /* duplicate number */
      *gameptr++ = (unsigned char) indx;
      i++;
      } /* for each number on ticket */
   ticketmm = atoi(*(argv+i));
   if (ticketmm < 1 || ticketmm > MEGABALL)
      {
      fprintf(stderr,"megamil: mega ball number %s "
         "is invalid\n",
         *(argv+i));
      putstx(*argv);
      } /* if invalid mega ball */
   p = (unsigned char *) ticket + ticketmm;
   if (*p == 255)
      {
      fprintf(stderr,"megamil: mega ball number %s "
         "is a duplicate\n",
         *(argv+i));
      putstx(*argv);
      } /* if duplicate mega ball */
   /* one game for $1.00 */
   /* twice/week for 100 years */
   /* 52 * 2 * 100 = 10400 */
   /* Based on Julian days, it computes to 10436 */
   /* ((2488434.5 - 2451910.5) / 7) * 2 = 10436 */ 
   maxplay = CENTURY;
   sd = (unsigned char *) rndinit();
   prizeptr = (double *) prize;
   *prizeptr++ = 5000000.00;
   *prizeptr++ = 175000.00;
   *prizeptr++ = 5000.00;
   *prizeptr++ = 150.00;
   *prizeptr++ = 150.00;
   *prizeptr++ = 10.00;
   *prizeptr++ = 7.00;
   *prizeptr++ = 3.00;
   *prizeptr   = 2.00;
   printf("Mega Millions ticket:  ");
   gameptr = (unsigned char *) game;
   i = FIVE;
   while (i--)
      {
      printf("%2d ", (int) *gameptr++);
      } /* first 5 numbers on ticket */
   printf("- %2d\n\n", ticketmm);
   printf("Match      Wins          Prize    "
      "   Expected           Odds\n");
   bingoptr = (double *) bingo;
   i = 9;
   while (i--) *bingoptr++ = 0.0;
   bingoptr = (double *) bingo;
   totplay = maxplay;
   while (totplay--)
      {
      p = (unsigned char *) fiftytwo;
      q = (unsigned char *) fiftytwo + FIFTYTWO;
      i = 0;
      while (p < q)
         *p++ = (unsigned char) ++i;
      *p = (unsigned char) '\0';
      totmtch = winmm = 0;
      p = (unsigned char *) fiftytwo;
      q = (unsigned char *) fiftytwo + FIFTYTWO - 1;
      r = (unsigned char *) ticket;
      trialptr = (unsigned char *) trial;
      max52 = FIFTYTWO;
      i = FIVE;
      while (i--)
         {
         indx = rndnum(max52,sd);
         tmp = (int) *(p+indx);
         *trialptr++ = (unsigned char) tmp;
         if (*(r+tmp) == 255) totmtch++;
         *(p+indx) = *q;
         *q-- = '\0';
         max52--;
         } /* pick 5 balls without replacement */
      winmm = rndnum(MEGABALL,sd) + 1;
      if (winmm != ticketmm) winmm = 0;
      won = 999999;
      if (totmtch == FIVE && winmm)
         won = 0;
      else if (totmtch == FIVE)
         won = 1;
      else if (totmtch == 4 && winmm)
         won = 2;
      else if (totmtch == 4)
         won = 3;
      else if (totmtch == 3 && winmm)
         won = 4;
      else if (totmtch == 3)
         won = 5;
      else if (totmtch == 2 && winmm)
         won = 6;
      else if (totmtch == 1 && winmm)
         won = 7;
      else if (totmtch == 0 && winmm)
         won = 8;
      else won = 9;
      *(bingoptr+won) += 1.0;
      } /* tst loop maxplay times */

   bingoptr = (double *) bingo;
   prizeptr = (double *) prize;
   dblplay = (double) maxplay;
   totprize = 0.0;

   /* grand prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getmm(5,1);
   expected = dblplay / prob;
   printf("5 1   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* 2nd prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getmm(5,0);
   expected = dblplay / prob;
   printf("5 0   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* 3rd prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getmm(4,1);
   expected = dblplay / prob;
   printf("4 1   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* 4th prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getmm(4,0);
   expected = dblplay / prob;
   printf("4 0   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* 5th prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getmm(3,1);
   expected = dblplay / prob;
   printf("3 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* 6th prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getmm(3,0);
   expected = dblplay / prob;
   printf("3 0   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* 7th prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getmm(2,1);
   expected = dblplay / prob;
   printf("2 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* 8th prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getmm(1,1);
   expected = dblplay / prob;
   printf("1 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* 9th prize */
   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getmm(0,1);
   expected = dblplay / prob;
   printf("0 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   bingoptr = (double *) bingo;
   sum = totbingo = totprob = 0.0;

   /* grand prize */
   odds = getmm(5,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   /* 2nd prize */
   odds = getmm(5,0);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   /* 3rd prize */
   odds = getmm(4,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   /* 4th prize */
   odds = getmm(4,0);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   /* 5th prize */
   odds = getmm(3,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   /* 6th prize */
   odds = getmm(3,0);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   /* 7th prize */
   odds = getmm(2,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   /* 8th prize */
   odds = getmm(1,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   /* 9th prize */
   odds = getmm(0,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;

   /* no prize */
   sum += ((dblplay - totbingo)
      * (dblplay - totbingo)
      / (1.0 - totprob));

   expected = dblplay / (1.0 / (1.0 - totprob));
   printf("0 0   %9.0f  %13.2f  %13.2f   %21.10f\n",
      (dblplay - totbingo),
      0.0,
      expected,
      (1.0 / (1.0 - totprob)));
   printf("        -------      ---------\n");
   printf("Total %9.0f  %13.2f\n\n",
      dblplay, totprize);
   printf("Total Cost       %13.2f\n\n",
      dblplay);
   netearn = totprize - dblplay;
   printf("Net Income       %13.2f\n\n",
      netearn);
   chisq = (sum / dblplay) - dblplay;
   printf("Chi-square %f\n", chisq);
   printf("Range at 95 percent "
      "is 2.70039 to 19.0228\n");
   free(sd);
   return(0);
   } /* main */
